      subroutine return_to_unit_triangle(theta, phi, omega, l_degree)

C     Checks whether or not the input orientation (theta, phi, omega)
C     lies within the icosahedral ASU defined by the triangle formed
C     from two adjacent points of fivefold symmetry (90, +/- 31.7) and
C     the nearest three-fold axis (69.0, 0).
C
C     For orientations outside of the ASU, the equivalent point within
C     the ASU is determined by individually applying the 60 rotation
C     matrices.
C      
C     Author: Steve Fuller 12/4/85
C     
C     MRC conversion sdf - 17-jan-87
C     Revision for larger area of return sdf 28-jan-87
C     Modified 1-feb-87 sdf
C     Completely re-written - previous version was wrong sdf 12-jan88
C     
C     Modified: Robert Sinkovits 6/28/06 - general cleanup and bringing
C     in line with Fortran 90 standard

      implicit none

      real*4, intent(inout) :: theta, phi, omega
      logical, intent(in)   :: l_degree

      real*4 dcos(3), x(3)
      real*4 thetar, phir, omegar
      real*4 deg_per_rad/57.29577951/
      real*4 icos_rot(3,3,60)   ! icos rotation matrixes
      real*4 three_fold(3)      ! direction cosines for three fold
      real*4 five_fold(3)       ! direction cosines for five fold
      real*4 ang_35,ang_55      ! angular distances btw 3fold and 5 folds
      real*4 area_std           ! area of asymmetric unit in spherical measure
      
      integer imat
      logical good, l_geometry_init

      common/icos_geometry/ l_geometry_init,
     1     icos_rot, three_fold, five_fold, ang_35, ang_55, area_std
      
      if(l_degree) then
         call polar_to_cosines(theta, phi, dcos)
      else
         call polar_to_cosines(theta*deg_per_rad,phi*deg_per_rad,dcos)
      end if
      
      if(good(dcos)) return
      
      if(.not.l_degree) then
         theta = theta * deg_per_rad
         phi = phi * deg_per_rad
         omega = omega * deg_per_rad
      else
         call polar_to_cosines( thetar*deg_per_rad,
     1        phir*deg_per_rad,x)          
      end if

      do imat = 1, 60
         call equivalent_view(theta, phi,omega,
     1        icos_rot(1,1,imat),
     1        thetar, phir, omegar)
         call polar_to_cosines(thetar, phir, x)
         if(good(x)) then
            if(l_degree) then
               theta = thetar
               phi = phir
               omega=omegar
               return
            else
               theta = thetar / deg_per_rad
               phi = phir / deg_per_rad
               omega = omegar / deg_per_rad
               return
            end if
         end if
      end do
      
      write(6,'('' return_to_unit_triangle '')')
      write(6,'('' no equivalent found '')')
      write(6,'('' input values '')')
      write(6,'('' theta '',f10.3)') theta
      write(6,'('' phi '',f10.3)') phi
      write(6,'('' omega '',f10.3)') omega
      write(6,'('' last matrix tried '',i2)') imat
      
      return
      end subroutine return_to_unit_triangle


      subroutine to_n22 (theta, phi, omega, z_sym)

C     Checks if orientation is within the specified n2 or n22 asymmetric
C     unit (ASU). If not, the equivalent position in the ASU is
C     calculated.
C     
C     geometric convention:
C     --------------------
C     In a right-handed x-y-z coordinate system, phi is a rotation about
C     the z-axis and theta is a rotation about the "new" y-axis (after
C     the phi rotation). Omega is a rotation about the normal
C     (determined by the theta, phi orientation).
C     
C     In this routine, the n-fold symmetry component is about the
C     z-axis. The value of n is input as z_sym. There is one 2-fold
C     symmetry axis about the x axis. This is assumed in the routine.
C     (for n22 symmetries there also is a 2-fold symmetry axis on the y
C     axis.) The minimum and maximum angles of phi (in the asymmetric
C     unit) are calculated from z_sym as phi_min and phi_max. phi_min
C     and phi_max differ only in sign, with the midpoint being phi =
C     0.0.  for all n2 and n22 asymmetric units, the minimum and maximum
C     theta angles are 0 and 90, respectively.
C     
C     For example, if you have 822 symmetry then the 8-fold symmetry
C     axis is defined along the z axis.  two two-fold axes are along
C     the x and y axes.  the 822 asymmetric unit is, in theta, 0 to 90
C     degrees, and in phi, -22.5 to +22.5 degrees.
C     
C     see belnap et al. (1997) j. struct. biol. 120:44-51 for a
C     definition of theta, phi, and omega angles.
C     
C     Purdue university dec 3000, 4 january 1994, D. Belnap
C     Added phi_dimension, 31 jan 1997, D. Belnap                            
C     General cleanup, 6/28/06, R Sinkovits

      implicit none

      integer*4, intent(in) :: z_sym
      real*4, intent(inout) :: theta, phi, omega

      real*4 phi_dimension, phi_min, phi_max

C     Calculate phi dimension of asymmetric unit
      phi_dimension = 360.0 / float(z_sym)

C     Calculate phi_min and phi_max
      phi_min  =  0.0  -  ( phi_dimension/2.0 )
      phi_max  =  0.0  +  ( phi_dimension/2.0 )

C     If theta at pole, phi = 0.0 and omega ranges between phi_min &
C     phi_max. Add or subtract phi_dimension if omega outside range
      if (theta.eq.0.0 .or. theta.eq.180.0) then
         phi = 0.0
         if (theta.eq.180.0) then
            theta = 0.0
            omega = -omega
         endif
         do while (omega.lt.phi_min)
            omega = omega + phi_dimension
         enddo
         do while (omega.gt.phi_max)
            omega = omega - phi_dimension
         enddo
         return
      endif

C     Determine if orientation is in ASU, exit if it is
      if (theta.le.90.0
     1     .and. phi.ge.phi_min .and. phi.le.phi_max)  return

C     Return stray orientations to ASU by adding or subtracting
C     phi_dimension
      do while (phi.lt.phi_min)
        phi = phi + phi_dimension
      enddo
      do while (phi.gt.phi_max)
        phi = phi - phi_dimension
      enddo

C     Return stray orientations to ASU by adjusting theta, phi, omega
      if (theta.gt.90.0) then
         theta = 180.0 - theta
         phi   = -phi
         omega = omega + 180.0
         if (omega.ge.360.0) omega = omega - 360.0
      endif
      
      return
      end subroutine to_n22


      subroutine to_nfold (theta, phi, omega, z_sym)
C
C     Tests whether of not a particular orientation falls within the
C     n-fold asymmetric unit (ASU), where the n-fold symmetry component
C     is about the z-axis. If the orientation does not fall within the
C     ASU, it's equivalent position in the ASU is calculated.  See the
C     description for subroutine to_n22 for geometric conventions.
C                                                                             
C     The value of n is input as z_sym and no other symmetry axes are
C     considered. The minimum and maximum angles of phi (in the ASU) are
C     calculated from z_sym as phi_min and phi_max.  phi_min and phi_max
C     differ only in sign, with the midpoint being phi = 0 for all
C     n-fold asymmetric units. The minimum and maximum theta angles are
C     0 and 180, respectively.
C                                                                             
C     For example, given 8-fold symmetry, the asymmetric unit is defined
C     by (0 <= theta <= 180) and |phi| <= 22.5 
C                                                                             
C     31 jan 1997, D. Belnap, modified from to_n22
C                                                                             
C     Modified 19-oct-2000 by TSB/MZ (check for omega > 360.0).
C     Modified 14-aug-2003 by MBS (theta -> 0 - 180 range).
C     Modified 28-jun-2006 by R Sinkovits - general cleanup

      implicit none

      integer*4, intent(in) :: z_sym
      real*4, intent(inout) :: theta, phi, omega

      real*4  phi_dimension, phi_min, phi_max

c     Bring theta into interval (0, 180); adjust phi accordingly
      if (theta .lt. 0.0) then
         theta = -theta
         phi   = phi + 180.
      end if
      if (theta .ge. 360.0) theta = amod(theta,360.0)
      if(theta .ge. 180.0) then
         theta = 360.  - theta
         phi   = phi   + 180.
      end if

C     Calculate phi dimension of asymmetric unit
      phi_dimension = 360.0 / float(z_sym)

C     Calculate phi_min and phi_max
      phi_min  =  0.0  -  ( phi_dimension/2.0 )
      phi_max  =  0.0  +  ( phi_dimension/2.0 )

C     If theta at pole, phi = 0.0 and omega ranges between phi_min &
C     phi_max. Add or subtract phi_dimension if omega outside range
      if (theta .eq. 0.0 .or. theta .eq. 180.0) then
         phi = 0.0
         do while (omega .lt. phi_min)
            omega = omega + phi_dimension
         enddo
         do while (omega .gt. phi_max)
            omega = omega - phi_dimension
         enddo
         return
      endif
      
C     Bring omega into interval (0, 360)
      if(omega .ge. 360.0) omega = amod(omega,360.0)

C     Determine if orientation is in ASU, exit if it is
      if (phi.ge.phi_min .and. phi.le.phi_max)  return

C     Return stray orientations to ASU by adding or subtracting
C     phi_dimension
      do while (phi.lt.phi_min)
        phi = phi + phi_dimension
      enddo
      do while (phi.gt.phi_max)
        phi = phi - phi_dimension
      enddo

      return
      end subroutine to_nfold


      subroutine to_asym_unit (symmetry, theta, phi, omega)
C
C     Calls routines to check if orientations are in the asymmetric
C     unit. If not, these routines calculate the equivalent view within
C     the asymmetric unit. For omega < 0, add 360 degrees; for phi > 360,
C     subtract 360 degrees.
C
C     Purdue University dec 3000, jan,jun 1994, D. Belnap
C     Added 222, 522, 622, & 722 symmetry, dec 1996, jan 1997, D. Belnap
C     Modified 22-jun-1998 by TSB (updated dihedral symmetries)
C     Modified 29-jun-1999 by D. Belnap (updated dihedral symmetries)
C     Modified 6/28/06 by R. Sinkovits - general cleanup

      implicit none

      integer*4, intent(in) :: symmetry
      real*4, intent(inout)   :: theta, phi, omega

      if (symmetry.ge.2 .and. symmetry.le.31) then ! n-fold symmetries
         call to_nfold (theta, phi, omega, symmetry)
      else if (symmetry.eq.532) then
         call return_to_unit_triangle (theta, phi, omega, .true.)
      else if (symmetry.eq.222) then
         call to_n22 (theta, phi, omega, 2)
      else if (symmetry.eq.32) then
         call to_n22 (theta, phi, omega, 3)
      else if (symmetry.eq.422) then
         call to_n22 (theta, phi, omega, 4)
      else if (symmetry.eq.52) then
         call to_n22 (theta, phi, omega, 5)
      else if (symmetry.eq.622) then
         call to_n22 (theta, phi, omega, 6)
      else if (symmetry.eq.72) then
         call to_n22 (theta, phi, omega, 7)
      else if (symmetry.eq.822) then
         call to_n22 (theta, phi, omega, 8)
      else if (symmetry.eq.92) then
         call to_n22 (theta, phi, omega, 9)
      endif
      
      return
      end subroutine to_asym_unit


      subroutine polar_to_cosines(theta, phi, x)
C
C     Convert (theta, phi, omega), given in degrees, to direction
C     cosines. A tolerance is set to try to alleviate round off problems
C     and the total length of the direction cosine vector is normalized
C     to 1.0
C
C     Author: Steve Fuller 1/10/88
C
C     Modified: Robert Sinkovits 6/28/06 - general cleanup and bringing
C     in line with Fortran 90 standard

      implicit none

      real*4, intent(in)  :: theta,phi
      real*4, intent(out) :: x(3)

      real*4 toler/1.0e-8/
      real*8 deg_to_rad
      integer i

      deg_to_rad = acos(-1.0d0)/180.0d0

      x(3) = cos(theta*deg_to_rad)

      if(abs(x(3)-1.0) .lt. toler) then
          x(2) = sin(phi*deg_to_rad)
          x(1) = sin(phi*deg_to_rad)
      else
          x(2) = sin(theta*deg_to_rad) * sin(phi*deg_to_rad)
          x(1) = sin(theta*deg_to_rad) * cos(phi*deg_to_rad)
      end if

      do i = 1,3
         if(abs(x(i)) .lt. toler) x(i)=0.0
      end do

      call norm(x)

      return
      end subroutine polar_to_cosines


      subroutine equivalent_view(theta, phi, omega, rot, 
     1     thetar, phir, omegar)
C
C     Find the (theta, phi, omega) for an equivalent view generated by
C     application of rotation matrix. New formulation which avoids the
C     cummulative errors in the many multiplications of the previous
C     version formulation suggested by Peter Metcalf
C
C     Author: Steve Fuller 10/29/89
C
C     Modified: Robert Sinkovits 6/28/06 - general cleanup and bringing
C     in line with Fortran 90 standard

      implicit none
      
      real*4, intent(in)  :: rot(3,3)             ! Rotation matrix
      real*4, intent(in)  :: theta, phi, omega    ! Original angles
      real*4, intent(out) :: thetar, phir, omegar ! Rotated angles

      real*4 view(3,3) ! Equivalent rotation matrix to crowther angles
      real*4 prot(3,3) ! Product matrix

      call crowther_to_matrix(theta, phi, omega, view)
      prot = matmul(view,rot) ! F90 matmul intrinsic
      call matrix_to_crowther(thetar, phir, omegar, prot)

      return
      end subroutine equivalent_view


      subroutine geometry_init
C
C     Initializes the arrays and geometric constants required by the
C     subroutine return_to_unit_triangle(). The variable l_geometry_init
C     forces geometry_init to be called the first time
C     return_to_unit_triangle is called.
C
C     Author: Steve Fuller 1/28/88
C
C     Modified: Robert Sinkovits 6/28/06 - general cleanup and bringing
C     in line with Fortran 90 standard

      implicit none

      integer i
      logical l_geometry_init      

      real*4 icos_rot(3,3,60)   ! icos rotation matrixes      
      real*4 three_fold(3)      ! direction cosines for three fold
      real*4 five_fold(3)       ! direction cosines for five fold
      real*4 ang_35,ang_55      ! angular distances btw 3fold and 5 folds
      real*4 area_std           ! area of asymmetric unit in spherical measure
      real*8 deg_to_rad, rad_to_deg
      real*8 tau, dist_35, dist_55
      real*8 theta_three, phi_five
      real*8 arg

      common/icos_geometry/ l_geometry_init,
     1      icos_rot, three_fold, five_fold, ang_35, ang_55, area_std

      deg_to_rad = acos(-1.0d0)/180.0d0
      rad_to_deg = 1.0d0/deg_to_rad

      call genrot(icos_rot)
      tau = (1.+sqrt(5.0d0))/2.0
      arg =  (1.+tau*tau/3.d0-1./(3.d0*tau*tau))/(2.d0*tau/sqrt(3.0d0))
      theta_three = 90. - rad_to_deg * acos(arg)
      three_fold(1) = sin(theta_three*deg_to_rad)
      three_fold(2) = 0.0
      three_fold(3) = cos(theta_three*deg_to_rad)

      phi_five = rad_to_deg * atan(1.0/tau)

      five_fold(1) = cos(phi_five*deg_to_rad)
      five_fold(2) = sin(phi_five*deg_to_rad)
      five_fold(3) = 0.0      

      dist_35 = 0.0

      do i = 1,3
         dist_35=(three_fold(i)-five_fold(i))**2 + dist_35
      end do

      dist_35 = sqrt(dist_35)
      dist_55 = five_fold(2)*2.d0

      ang_55=2.0*asin(dist_55/2.)
      ang_35=2.0*asin(dist_35/2.)
      
      area_std  = 4*3.141592653589793/60. ! 1/60th sphere surface

      l_geometry_init = .true.

      return
      end subroutine geometry_init


      logical function good(x)
C
C     Logical function good() returns the value TRUE if the orientation
C     specified by the direction cosine vector falls within the
C     icosahedral asymmetric unit (ASU). The outer surface of the ASU is
C     defined by the triangle formed from two adjacent points of
C     fivefold symmetry (90, +/- 31.7) and the nearest three-fold axis
C     (69.0, 0) The function does a rough check first on the direction
C     cosines of the view and then calculates the spherical measure of
C     the largest triangle formed by the test point and the bounding
C     symmetry axes. If this is larger than that of the asymmetric unit,
C     the view is outside the unit.
C
C     Author: Steve Fuller 10/8/88
C
C     Modified: Robert Sinkovits 6/28/06 - general cleanup and bringing
C     in line with Fortran 90 standard

      implicit none

      real*4, intent(in) :: x(3)

      real*4 toler/1.0e-6/
      real*4 a3x, a5px, a5x, d3x, d5x, d5px, area_x
      real*4 three_fold(3)      ! direction cosines for three fold
      real*4 five_fold(3)       ! direction cosines for five fold
      real*4 ang_35,ang_55      ! angular distances btw 3fold and 5 folds
      real*4 area_std           ! area of asymmetric unit in spherical measure
      real*4 icos_rot(3,3,60)   ! icos rotation matrixes
      real*4 sur,excess,a,b,c

      integer i
      logical l_geometry_init      
      
      common/icos_geometry/ l_geometry_init,
     1      icos_rot, three_fold, five_fold, ang_35, ang_55, area_std

C     Arithmetic function definitions
      sur(a,b,c) = (a+b+c)/2.0
      excess(a,b,c) =
     1     4.0 * atan(sqrt(abs(tan(sur(a,b,c)/2.)
     1     * tan((sur(a,b,c)-a)/2.0)
     1     * tan((sur(a,b,c)-b)/2.0)
     1     * tan((sur(a,b,c)-c)/2.0) ) )  )
      
      good = .false.

      if(.not. l_geometry_init) then
         call geometry_init
      endif

      if(x(3)        .lt. -toler .or.
     1     x(3)      .gt. three_fold(3)+toler .or.
     2     abs(x(2)) .gt. five_fold(2)+toler) then
         return
      end if

      d3x = 0.0
      d5x = 0.0
      d5px = 0.0

      do i = 1,3
         d3x=(x(i)-three_fold(i))**2 + d3x
         d5x=(x(i)-five_fold(i))**2 + d5x
         if(i .eq. 2) then
            d5px=(x(i)+five_fold(i))**2 + d5px
         else
            d5px=(x(i)-five_fold(i))**2 + d5px
         end if
      end do
      
      d3x = sqrt(d3x)
      d5x = sqrt(d5x)
      d5px = sqrt(d5px)
      
      a5x=2.0*asin(d5x/2.)
      a5px=2.0*asin(d5px/2.)
      a3x=2.0*asin(d3x/2.)
      
      area_x = excess(a5x,a3x,ang_35) +
     1     excess(a5px,a3x,ang_35) +
     1     excess(a5x,a5px,ang_55) -
     1     min(excess(a5px,a3x,ang_35),excess(a5x,a5px,ang_55), 
     1     excess(a5x,a3x,ang_35))

      good = area_x .lt. (area_std+toler)

      return
      end function good


      subroutine norm(x)
C     
C     Normalizes a three-element vector and returns the normalized value
C
C     Author: Steve Fuller 1/8/88
C
C     Modified: Robert Sinkovits 6/28/06 - general cleanup and bringing
C     in line with Fortran 90 standard

      implicit none
      
      real, intent(inout) :: x(3)
      real  ndot
      
      ndot = sqrt(x(1)*x(1) + x(2)*x(2) + x(3)*x(3))
      if(ndot .gt. 1.0e-20) then
         x = x / ndot ! F90 array syntax
      else
         print *, 'Subroutine norm - zero length vector ', x
         x = 0.0 ! F90 array syntax
      endif

      return
      end subroutine norm


      subroutine genrot(rot)                              
C
C     Modified from G. Bricogne's program genrot in the mrc package.
C     Generates the 60 icosahedral rotations and their matrices
C     expressed in the orthnormal frame based on three dyads .
C
C     Author: Steve Fuller 10/29/89
C
C     Modified: Robert Sinkovits 6/27/06 - general cleanup and bringing
C     in line with Fortran 90 standard

      implicit real*8 (a-h,o-z)

      real*4, intent(out) :: rot(3,3,60)

      real*8 drot(3,3)
      real*8 p3(3,3)/9*0.0d0/,p5(3,3)/9*0.0d0/
      real*8 p3inv(3,3)/9*0.0d0/,p5inv(3,3)/9*0.0d0/,tmat6(3,3)
      real*8 tmat1(3,3),tmat2(3,3),tmat3(3,3),tmat4(3,3),tmat5(3,3)
      real*8 r2y(3,3,2)/18*0.0d0/,r2x(3,3,2)/18*0.0d0/
      real*8 s3(3,3,3)/27*0.0d0/,s5(3,3,5)/45*0.0d0/

      pi     = acos(-1.0d0)
      tpiov3 = pi/1.5
      tpiov5 = pi/2.5

      root5 = sqrt(5.0)

      xx3  = root5/6.0
      cos3 = sqrt(0.5+xx3)
      sin3 = sqrt(0.5-xx3)

      xx5  = 0.5/root5
      cos5 = sqrt(0.5+xx5)
      sin5 = sqrt(0.5-xx5)

C     Generate y-dyad matrices
      do k = 1,3
         r2y(k,k,1) = 1.0
      enddo
      r2y(1,1,2) = -1.0
      r2y(2,2,2) =  1.0
      r2y(3,3,2) = -1.0

C     Generate 3-fold matrices
      theta = -tpiov3
      do k = 1,3
         theta = theta + tpiov3
         cosine = cos(theta)
         sine = sin(theta)
         s3(1,1,k) = 1.0
         s3(2,2,k) = cosine
         s3(3,2,k) = sine
         s3(2,3,k) = -sine
         s3(3,3,k) = cosine
      enddo

C     Generate 5-fold matrices
      theta = -tpiov5
      do k = 1,5
         theta  = theta+tpiov5
         cosine = cos(theta)
         sine   = sin(theta)
         s5(1,1,k) = cosine
         s5(2,1,k) = sine
         s5(1,2,k) = -sine
         s5(2,2,k) = cosine
         s5(3,3,k) = 1.0
      enddo

C     Generate x-dyad matrices
      do k = 1,3
         r2x(k,k,1) = 1.0
      enddo
      r2x(1,1,2) = 1.0
      r2x(2,2,2) = -1.0
      r2x(3,3,2) = -1.0

C     Generate  xz-threefold transition matrices
      p3(1,1) = cos3
      p3(3,1) = sin3
      p3(2,2) = 1.0
      p3(1,3) = -sin3
      p3(3,3) = cos3
      p3inv(1,1) = cos3
      p3inv(3,1) = -sin3
      p3inv(2,2) = 1.0
      p3inv(1,3) = sin3
      p3inv(3,3) = cos3

C     Generate  xz-fivefold transition matrices
      p5(1,1) = cos5
      p5(3,1) = sin5
      p5(2,2) = 1.0
      p5(1,3) = -sin5
      p5(3,3) = cos5
      p5inv(1,1) = cos5
      p5inv(3,1) = -sin5
      p5inv(2,2) = 1.0
      p5inv(1,3) = sin5
      p5inv(3,3) = cos5

C     Generate icosahedral rotation matrices. Note use of Fortran 90
C     array syntax and matmul intrinsic function.
      n = 0
      do l2 = 1,2
         tmat1 = matmul(r2y(1:3,1:3,l2),p5)
         do k5 = 1,5
            tmat2 = matmul(tmat1,s5(1:3,1:3,k5))
            tmat3 = matmul(tmat2,p5inv)
            do j2 = 1,2
               tmat4 = matmul(tmat3,r2x(1:3,1:3,j2))
               tmat5 = matmul(tmat4,p3)
               do i3 = 1,3
                  n = n+1
                  tmat6 = matmul(tmat5,s3(1:3,1:3,i3))
                  drot = matmul(tmat6,p3inv)
                  rot(1:3,1:3,n) = drot(1:3,1:3)
               enddo
            enddo
         enddo
      enddo
      
      return
      end subroutine genrot


      subroutine matrix_to_crowther(theta, phi, omega, rot)
C
C     Returns the Crowther angles (theta, phi, omega) corresponding to a
C     rotation matrix.
C          
C     Peter's derivation is based on crowther matrix as:
C
C     so*ct*cp - co*sp       -so*ct*sp - co*cp          -so*st
C     co*ct*cp + so*sp       -co*ct*sp + so*cp          -co*st
C     st*cp                  -st*sp                      ct 
C
C     Author: Steve Fuller 10/29/89
C
C     Modified: Robert Sinkovits 6/27/06 - general cleanup and
C     replacement of nonstandard acosd()/atan2d() functions with Fortran
C     90 acos()/atan2()

      implicit none

      real*4, intent(inout)  :: rot(3,3)
      real*4, intent(out) :: theta, phi, omega

      real*4 omega_minus_phi, omega_plus_phi
      real*4 toler/1.0e-6/
      real*8 deg_to_rad, rad_to_deg

      deg_to_rad = acos(-1.0d0)/180.0d0
      rad_to_deg = 1.0d0/deg_to_rad

C     Fix rotation matrix if not accurate by this method - bug fix
      if(abs(rot(3,3))+toler .ge. 1.0) then
          rot(3,3)=sign(1.0, rot(3,3))
      endif                                             
            
      theta = rad_to_deg*acos(rot(3,3))

C     If theta is very close to zero, then there is flexibility in the
C     assignment of the angles omega and phi. In this case, arbitrarily
C     set omega to zero. Note also that all minus signs in atan2() are
C     required.

      if(sin(theta*deg_to_rad) .gt. toler) then
         omega = rad_to_deg * atan2(-rot(1,3),-rot(2,3))
         phi =   rad_to_deg * atan2(-rot(3,2),rot(3,1))
      else  
         omega=0.0
         if(cos(theta*deg_to_rad) .eq. 1.0) then
            omega_minus_phi = rad_to_deg * atan2(rot(2,2),rot(2,1))
            phi = omega - omega_minus_phi
         else
            omega_plus_phi= rad_to_deg * atan2(rot(2,2),-rot(2,1))
            phi=omega_plus_phi
         endif
      endif            

      return
      end subroutine matrix_to_crowther


      subroutine crowther_to_matrix(theta, phi, omega, rot)
C
C     Generates the rotation matrix corresponding to the Crowther angles
C     (theta, phi, omega)
C
C     Author: Steve Fuller 10/29/89
C
C     Modified: Robert Sinkovits 6/27/06 - general cleanup and
C     replacement of nonstandard sind()/cosd() functions with Fortran 90
C     sin()/cos()

      implicit none
      real*4, intent(out) :: rot(3,3)
      real*4, intent(in)  :: theta, phi, omega
      real*4 ct, st, cp, sp, co, so
      real*8 deg_to_rad

      deg_to_rad = acos(-1.0d0)/180.0d0

      st=sin(theta*deg_to_rad)
      ct=cos(theta*deg_to_rad)
      sp=sin(phi*deg_to_rad)
      cp=cos(phi*deg_to_rad)
      so=sin(omega*deg_to_rad)
      co=cos(omega*deg_to_rad)

      rot(1,1)= so*ct*cp - co*sp
      rot(1,2)=-so*ct*sp - co*cp
      rot(1,3)=-so*st
      rot(2,1)= co*ct*cp + so*sp
      rot(2,2)=-co*ct*sp + so*cp
      rot(2,3)=-co*st
      rot(3,1)= st*cp
      rot(3,2)=-st*sp
      rot(3,3)= ct
 
      return
      end subroutine crowther_to_matrix
